@@ -29,10 +29,11 @@ def calc_metrics(y_pred, y_true, metrics_dic: OrderedDict):
     return metrics
 
 class MetricsRecorder():
-    def __init__(self, metrics, epoch_metrics) -> None:
+    def __init__(self, metrics, epoch_metrics, loop_config) -> None:
         self.metrics = metrics
         self.epoch_metrics = OrderedDict([(k+'#', v) for k,v in epoch_metrics.items()])
         self.has_epoch_metrics = len(self.epoch_metrics) > 0
+        self.loop = loop_config
 
         # for batch metrics
         self.accum_metrics = OrderedDict([(k, 0.0) for k in self.metrics.keys()])
@@ -45,8 +46,9 @@ class MetricsRecorder():
     def update(self, y_batch_pred, y_batch):
         self.__update_batch_metrics(y_batch_pred, y_batch)
         if self.has_epoch_metrics:
-            self.y_pred.append(y_batch_pred.cpu())
-            self.y_true.append(y_batch.cpu())
+            y_batch_pred, y_batch = self.loop.cache_for_epoch_metrics(y_batch_pred, y_batch)
+            self.y_pred.append(y_batch_pred)
+            self.y_true.append(y_batch)
 
     def __update_batch_metrics(self, y_batch_pred, y_batch):
         count = len(y_batch)
@@ -128,9 +130,9 @@ class Trainer():
             func(self)
 
     def get_loop_config(self, train):
-        loop = deepcopy(self.loop_config)
-        loop._train = train
-        return loop
+        self.loop_config._train = train
+        self.loop_config.on_epoch_begin()
+        return self.loop_config
 
     def run(self, train_loader, val_loader, max_epochs, verbose, use_amp, accum_grad_steps):
         self.data_loaders = train_loader, val_loader
@@ -163,8 +165,8 @@ class Trainer():
 
     def train_on_epoch(self, data_loader):
         self.model.train()
-        metrics_rec = MetricsRecorder(self.metrics, self.epoch_metrics)
         loop = self.get_loop_config(train=True)
+        metrics_rec = MetricsRecorder(self.metrics, self.epoch_metrics, loop)
 
         if self.use_amp:
             grad_scaler = torch.cuda.amp.GradScaler()
@@ -181,7 +183,12 @@ class Trainer():
         batch_idx, max_batch_idx = 0, len(data_loader)
         for batch in data_loader:
             self.__fire_event(Events.ON_BATCH_BEGIN)
-            batch = [i.to(device=self.device) for i in batch]
+
+            if isinstance(batch, list):
+                batch = [i.to(device=self.device) for i in batch]
+            else:
+                batch = batch.to(device=self.device)
+
             x_batch, y_batch = loop.process_batch(batch)
             
             with autocast(self.use_amp, self.device):
@@ -217,12 +224,19 @@ class Trainer():
     @torch.no_grad()
     def valid_on_epoch(self, data_loader, use_amp):
         self.model.eval()
-        metrics_rec = MetricsRecorder(self.metrics, self.epoch_metrics)
-
         loop = self.get_loop_config(train=False)
+
+        val_metrics = self.metrics.copy()
+        if self.disable_val_loss:
+            del val_metrics['loss']
+        metrics_rec = MetricsRecorder(val_metrics, self.epoch_metrics, loop)
  
         for batch in data_loader:
-            batch = [i.to(device=self.device) for i in batch]
+            if isinstance(batch, list):
+                batch = [i.to(device=self.device) for i in batch]
+            else:
+                batch = batch.to(device=self.device)
+
             x_batch, y_batch = loop.process_batch(batch)
 
             with autocast(use_amp, self.device):
