@@ -8,6 +8,9 @@ from ._util_layers import Lambda
 class KerasLayer(nn.Module):
     def __init__(self, *args, **kwargs) -> None:
         super(KerasLayer, self).__init__()
+
+        print('[Warning] This module is obsolete and may cause serious bugs. Please use `nn.modules.lazy.LazyModuleMixin` instead.')
+
         self.args = args
         self.kwargs = kwargs
         self.module = None
@@ -16,18 +19,18 @@ class KerasLayer(nn.Module):
     def build(self, in_shape: torch.Size):
         pass
 
-    def forward(self, x):
-        if not self.module:
+    def forward(self, x, *args, **kwargs):
+        if self.module is None:
             self.module = self.build(x.shape)
             self.module._k4t_layer_tag = 0
-        return self.module(x)
+        return self.module(x, *args, **kwargs)
 
     @property
     def is_built(self):
         return self.module is not None
 
     def apply(self, fn):
-        if not self.module:
+        if self.module is None:
             raise AssertionError("This module hasn't been built yet.")
         return self.module.apply(fn)
 
