@@ -1,6 +1,5 @@
 import torch
 import torch.nn as nn
-import torch.nn.functional as F
 
 class Lambda(nn.Module):
     """Wrap a function as `nn.Module`."""
@@ -74,9 +73,6 @@ class SamePadding(nn.Module):
 '''
 
 class Add(nn.Module):
-    def __init__(self):
-        super(Add, self).__init__()
-
     def forward(self, x):
         return sum(x)
 
